@@ -49,7 +49,7 @@ struct _CcColorPanelPrivate
 };
 
 enum {
-  GCM_PREFS_COLUMN_DEVICE_ID,
+  GCM_PREFS_COLUMN_DEVICE_PATH,
   GCM_PREFS_COLUMN_SORT,
   GCM_PREFS_COLUMN_ICON,
   GCM_PREFS_COLUMN_TITLE,
@@ -79,6 +79,9 @@ typedef enum {
 #define GCM_SETTINGS_RECALIBRATE_PRINTER_THRESHOLD      "recalibrate-printer-threshold"
 #define GCM_SETTINGS_RECALIBRATE_DISPLAY_THRESHOLD      "recalibrate-display-threshold"
 
+/* max number of devices and profiles to cause auto-expand at startup */
+#define GCM_PREFS_MAX_DEVICES_PROFILES_EXPANDED         5
+
 static void     gcm_prefs_device_add_cb (GtkWidget *widget, CcColorPanel *prefs);
 
 static void
@@ -198,7 +201,7 @@ gcm_prefs_file_chooser_get_icc_profile (CcColorPanel *prefs)
   dialog = gtk_file_chooser_dialog_new (_("Select ICC Profile File"), window,
                                         GTK_FILE_CHOOSER_ACTION_OPEN,
                                         GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-                                        _("Import"), GTK_RESPONSE_ACCEPT,
+                                        _("_Import"), GTK_RESPONSE_ACCEPT,
                                         NULL);
   gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER(dialog), g_get_home_dir ());
   gtk_file_chooser_set_create_folders (GTK_FILE_CHOOSER(dialog), FALSE);
@@ -249,6 +252,7 @@ gcm_prefs_calibrate_cb (GtkWidget *widget, CcColorPanel *prefs)
   g_ptr_array_add (argv, g_strdup (cd_device_get_id (priv->current_device)));
   g_ptr_array_add (argv, g_strdup ("--parent-window"));
   g_ptr_array_add (argv, g_strdup_printf ("%i", xid));
+  g_ptr_array_add (argv, NULL);
   ret = g_spawn_async (NULL, (gchar**) argv->pdata, NULL, 0,
                        NULL, NULL, NULL, &error);
   if (!ret)
@@ -347,6 +351,14 @@ gcm_prefs_combo_sort_func_cb (GtkTreeModel *model,
   return retval;
 }
 
+static gboolean
+gcm_prefs_combo_set_default_cb (gpointer user_data)
+{
+  GtkWidget *widget = GTK_WIDGET (user_data);
+  gtk_combo_box_set_active (GTK_COMBO_BOX (widget), 0);
+  return FALSE;
+}
+
 static void
 gcm_prefs_add_profiles_suitable_for_devices (CcColorPanel *prefs,
                                              GtkWidget *widget,
@@ -410,6 +422,12 @@ gcm_prefs_add_profiles_suitable_for_devices (CcColorPanel *prefs,
       if (!ret)
         continue;
 
+#if CD_CHECK_VERSION(0,1,13)
+      /* ignore profiles from other user accounts */
+      if (!cd_profile_has_access (profile_tmp))
+        continue;
+#endif
+
       /* add */
       gcm_prefs_combobox_add_profile (widget,
                                       profile_tmp,
@@ -418,8 +436,10 @@ gcm_prefs_add_profiles_suitable_for_devices (CcColorPanel *prefs,
     }
 
   /* add a import entry */
+#if CD_CHECK_VERSION(0,1,12)
   gcm_prefs_combobox_add_profile (widget, NULL, GCM_PREFS_ENTRY_TYPE_IMPORT, NULL);
-  gtk_combo_box_set_active (GTK_COMBO_BOX (widget), 0);
+#endif
+  g_idle_add (gcm_prefs_combo_set_default_cb, widget);
 out:
   if (profile_array != NULL)
     g_ptr_array_unref (profile_array);
@@ -462,7 +482,7 @@ gcm_prefs_profile_remove_cb (GtkWidget *widget, CcColorPanel *prefs)
                                                "treeview_devices"));
   selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (widget));
   if (!gtk_tree_selection_get_selected (selection, &model, &iter))
-    g_assert_not_reached ();
+    goto out;
 
   /* if the profile is default, then we'll have to make the first profile default */
   gtk_tree_model_get (model, &iter,
@@ -491,6 +511,7 @@ gcm_prefs_profile_make_default_internal (CcColorPanel *prefs,
                                          GtkTreeModel *model,
                                          GtkTreeIter *iter_selected)
 {
+  CdDevice *device;
   CdProfile *profile;
   GError *error = NULL;
   gboolean ret = FALSE;
@@ -498,13 +519,14 @@ gcm_prefs_profile_make_default_internal (CcColorPanel *prefs,
 
   /* get currentlt selected item */
   gtk_tree_model_get (model, iter_selected,
+                      GCM_PREFS_COLUMN_DEVICE, &device,
                       GCM_PREFS_COLUMN_PROFILE, &profile,
                       -1);
-  if (profile == NULL)
+  if (profile == NULL || device == NULL)
     goto out;
 
   /* just set it default */
-  ret = cd_device_make_profile_default_sync (priv->current_device,
+  ret = cd_device_make_profile_default_sync (device,
                                              profile,
                                              priv->cancellable,
                                              &error);
@@ -517,6 +539,8 @@ gcm_prefs_profile_make_default_internal (CcColorPanel *prefs,
 out:
   if (profile != NULL)
     g_object_unref (profile);
+  if (device != NULL)
+    g_object_unref (device);
 }
 
 static void
@@ -555,6 +579,7 @@ gcm_prefs_profile_view_cb (GtkWidget *widget, CcColorPanel *prefs)
   g_ptr_array_add (argv, g_strdup (cd_profile_get_id (profile)));
   g_ptr_array_add (argv, g_strdup ("--parent-window"));
   g_ptr_array_add (argv, g_strdup_printf ("%i", xid));
+  g_ptr_array_add (argv, NULL);
   ret = g_spawn_async (NULL, (gchar**) argv->pdata, NULL, 0,
                        NULL, NULL, NULL, &error);
   if (!ret)
@@ -604,6 +629,11 @@ gcm_prefs_button_assign_ok_cb (GtkWidget *widget, CcColorPanel *prefs)
   gtk_tree_model_get (model, &iter,
                       GCM_PREFS_COMBO_COLUMN_PROFILE, &profile,
                       -1);
+  if (profile == NULL)
+    {
+        g_warning ("failed to get the active profile");
+        goto out;
+    }
 
   /* just add it, the list store will get ::changed */
   ret = cd_device_add_profile_sync (priv->current_device,
@@ -866,6 +896,7 @@ gcm_prefs_profile_clicked (CcColorPanel *prefs, CdProfile *profile, CdDevice *de
 {
   GtkWidget *widget;
   CdDeviceRelation relation;
+  gchar *s;
   CcColorPanelPrivate *priv = prefs->priv;
 
   /* get profile */
@@ -896,7 +927,18 @@ gcm_prefs_profile_clicked (CcColorPanel *prefs, CdProfile *profile, CdDevice *de
   /* allow getting profile info */
   widget = GTK_WIDGET (gtk_builder_get_object (priv->builder,
                "toolbutton_profile_view"));
-  gtk_widget_set_sensitive (widget, TRUE);
+  if ((s = g_find_program_in_path ("gcm-viewer")))
+    {
+      gtk_widget_set_sensitive (widget, TRUE);
+      g_free (s);
+    }
+  else
+      gtk_widget_set_sensitive (widget, FALSE);
+
+  /* hide device specific stuff */
+  widget = GTK_WIDGET (gtk_builder_get_object (priv->builder,
+                                               "toolbutton_device_remove"));
+  gtk_widget_set_visible (widget, FALSE);
 }
 
 static void
@@ -933,7 +975,10 @@ gcm_prefs_devices_treeview_clicked_cb (GtkTreeSelection *selection,
   gtk_widget_set_visible (widget, FALSE);
   widget = GTK_WIDGET (gtk_builder_get_object (priv->builder,
                                                "toolbutton_device_calibrate"));
-  gtk_widget_set_visible (widget, profile == NULL);
+  gtk_widget_set_visible (widget, device != NULL);
+  widget = GTK_WIDGET (gtk_builder_get_object (priv->builder,
+                                               "toolbutton_profile_add"));
+  gtk_widget_set_visible (widget, device != NULL);
   widget = GTK_WIDGET (gtk_builder_get_object (priv->builder,
                                                "toolbutton_profile_view"));
   gtk_widget_set_visible (widget, profile != NULL);
@@ -941,6 +986,11 @@ gcm_prefs_devices_treeview_clicked_cb (GtkTreeSelection *selection,
                                                "toolbutton_profile_remove"));
   gtk_widget_set_visible (widget, profile != NULL);
 
+  /* if no buttons then hide toolbar */
+  widget = GTK_WIDGET (gtk_builder_get_object (priv->builder,
+                                               "toolbar_devices"));
+  gtk_widget_set_visible (widget, profile != NULL || device != NULL);
+
   if (device != NULL)
     g_object_unref (device);
   if (profile != NULL)
@@ -1052,6 +1102,7 @@ gcm_prefs_profile_combo_changed_cb (GtkWidget *widget,
                                     CcColorPanel *prefs)
 {
   GFile *file = NULL;
+  GError *error = NULL;
   gboolean ret;
   CdProfile *profile = NULL;
   GtkTreeIter iter;
@@ -1082,9 +1133,35 @@ gcm_prefs_profile_combo_changed_cb (GtkWidget *widget,
         {
           g_warning ("failed to get ICC file");
           gtk_combo_box_set_active (GTK_COMBO_BOX (widget), 0);
+
+          /* if we've got no other existing profiles to choose, then
+           * just close the assign dialog */
+          gtk_combo_box_get_active_iter (GTK_COMBO_BOX(widget), &iter);
+          gtk_tree_model_get (model, &iter,
+                              GCM_PREFS_COMBO_COLUMN_TYPE, &entry_type,
+                              -1);
+          if (entry_type == GCM_PREFS_ENTRY_TYPE_IMPORT)
+            {
+              widget = GTK_WIDGET (gtk_builder_get_object (priv->builder,
+                                                           "dialog_assign"));
+              gtk_widget_hide (widget);
+            }
           goto out;
         }
 
+#if CD_CHECK_VERSION(0,1,12)
+      profile = cd_client_import_profile_sync (priv->client,
+                                               file,
+                                               priv->cancellable,
+                                               &error);
+      if (profile == NULL)
+        {
+          g_warning ("failed to get imported profile: %s", error->message);
+          g_error_free (error);
+          goto out;
+        }
+#endif
+
       /* add to combobox */
       gtk_list_store_append (GTK_LIST_STORE(model), &iter);
       gtk_list_store_set (GTK_LIST_STORE(model), &iter,
@@ -1287,7 +1364,7 @@ gcm_prefs_get_profile_title (CcColorPanel *prefs, CdProfile *profile)
   title = cd_profile_get_title (profile);
   if (title != NULL)
     {
-      string = g_strdup (title);
+      string = g_markup_escape_text (title, -1);
       goto out;
     }
 
@@ -1332,7 +1409,7 @@ gcm_prefs_device_remove_profiles_phase1 (CcColorPanel *prefs, GtkTreeIter *paren
   /* mark to be removed */
   do {
     gtk_tree_store_set (priv->list_store_devices, &iter,
-                        GCM_PREFS_COLUMN_DEVICE_ID, NULL,
+                        GCM_PREFS_COLUMN_DEVICE_PATH, NULL,
                         -1);
   } while (gtk_tree_model_iter_next (model, &iter));
 }
@@ -1356,7 +1433,7 @@ gcm_prefs_device_remove_profiles_phase2 (CcColorPanel *prefs, GtkTreeIter *paren
   do
     {
       gtk_tree_model_get (model, &iter,
-              GCM_PREFS_COLUMN_DEVICE_ID, &id_tmp,
+              GCM_PREFS_COLUMN_DEVICE_PATH, &id_tmp,
               -1);
       if (id_tmp == NULL)
         ret = gtk_tree_store_remove (priv->list_store_devices, &iter);
@@ -1438,6 +1515,16 @@ gcm_prefs_device_set_model_by_iter (CcColorPanel *prefs, CdDevice *device, GtkTr
       goto out;
     }
 
+#if CD_CHECK_VERSION(0,1,13)
+      /* ignore profiles from other user accounts */
+      if (!cd_profile_has_access (profile))
+        {
+          g_warning ("%s is not usable by this user",
+                     cd_profile_get_filename (profile));
+          goto out;
+        }
+#endif
+
   /* autogenerated printer defaults */
   if (cd_device_get_kind (device) == CD_DEVICE_KIND_PRINTER &&
       cd_profile_get_filename (profile) == NULL)
@@ -1532,7 +1619,7 @@ skip:
       gtk_tree_store_set (priv->list_store_devices, iter_tmp_p,
                           GCM_PREFS_COLUMN_DEVICE, device,
                           GCM_PREFS_COLUMN_PROFILE, profile_tmp,
-                          GCM_PREFS_COLUMN_DEVICE_ID, cd_device_get_id (device),
+                          GCM_PREFS_COLUMN_DEVICE_PATH, cd_device_get_object_path (device),
                           GCM_PREFS_COLUMN_SORT, sort_tmp,
                           GCM_PREFS_COLUMN_STATUS, date_tmp->str,
                           GCM_PREFS_COLUMN_TITLE, title_tmp,
@@ -1558,11 +1645,13 @@ out:
 static void
 gcm_prefs_device_changed_cb (CdDevice *device, CcColorPanel *prefs)
 {
+  const gchar *id;
+  gboolean ret;
+  gchar *id_tmp;
   GtkTreeIter iter;
   GtkTreeModel *model;
-  const gchar *id;
-  gchar *id_tmp;
-  gboolean ret;
+  GtkTreeSelection *selection;
+  GtkWidget *widget;
   CcColorPanelPrivate *priv = prefs->priv;
 
   /* get first element */
@@ -1572,14 +1661,22 @@ gcm_prefs_device_changed_cb (CdDevice *device, CcColorPanel *prefs)
     return;
 
   /* get the other elements */
-  id = cd_device_get_id (device);
+  id = cd_device_get_object_path (device);
   do
     {
       gtk_tree_model_get (model, &iter,
-              GCM_PREFS_COLUMN_DEVICE_ID, &id_tmp,
-              -1);
+                          GCM_PREFS_COLUMN_DEVICE_PATH, &id_tmp,
+                          -1);
       if (g_strcmp0 (id_tmp, id) == 0)
-        gcm_prefs_device_set_model_by_iter (prefs, device, &iter);
+        {
+          /* populate device, and then select it */
+          gcm_prefs_device_set_model_by_iter (prefs, device, &iter);
+          widget = GTK_WIDGET (gtk_builder_get_object (priv->builder,
+                                                       "treeview_devices"));
+          selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (widget));
+          gtk_tree_selection_select_iter (selection, &iter);
+
+        }
       g_free (id_tmp);
     } while (gtk_tree_model_iter_next (model, &iter));
 }
@@ -1597,6 +1694,7 @@ gcm_prefs_add_device (CcColorPanel *prefs, CdDevice *device)
   GtkTreeIter parent;
   CcColorPanelPrivate *priv = prefs->priv;
 
+
   /* get device properties */
   ret = cd_device_connect_sync (device, priv->cancellable, &error);
   if (!ret)
@@ -1623,12 +1721,12 @@ gcm_prefs_add_device (CcColorPanel *prefs, CdDevice *device)
                     G_CALLBACK (gcm_prefs_device_changed_cb), prefs);
 
   /* add to list */
-  id = cd_device_get_id (device);
+  id = cd_device_get_object_path (device);
   g_debug ("add %s to device list", id);
   gtk_tree_store_append (priv->list_store_devices, &parent, NULL);
   gtk_tree_store_set (priv->list_store_devices, &parent,
                       GCM_PREFS_COLUMN_DEVICE, device,
-                      GCM_PREFS_COLUMN_DEVICE_ID, id,
+                      GCM_PREFS_COLUMN_DEVICE_PATH, id,
                       GCM_PREFS_COLUMN_SORT, sort,
                       GCM_PREFS_COLUMN_TITLE, title,
                       GCM_PREFS_COLUMN_ICON, icon_name,
@@ -1644,23 +1742,13 @@ gcm_prefs_remove_device (CcColorPanel *prefs, CdDevice *cd_device)
 {
   GtkTreeIter iter;
   GtkTreeModel *model;
-  GError *error = NULL;
   const gchar *id;
   gchar *id_tmp;
   gboolean ret;
   CcColorPanelPrivate *priv = prefs->priv;
 
-  /* get device properties */
-  ret = cd_device_connect_sync (cd_device, priv->cancellable, &error);
-  if (!ret)
-    {
-      g_warning ("failed to connect to the device: %s", error->message);
-      g_error_free (error);
-      return;
-    }
-
   /* remove */
-  id = cd_device_get_id (cd_device);
+  id = cd_device_get_object_path (cd_device);
 
   /* get first element */
   model = GTK_TREE_MODEL (priv->list_store_devices);
@@ -1672,7 +1760,7 @@ gcm_prefs_remove_device (CcColorPanel *prefs, CdDevice *cd_device)
   do
     {
       gtk_tree_model_get (model, &iter,
-                          GCM_PREFS_COLUMN_DEVICE_ID, &id_tmp,
+                          GCM_PREFS_COLUMN_DEVICE_PATH, &id_tmp,
                           -1);
       if (g_strcmp0 (id_tmp, id) == 0)
         {
@@ -1684,16 +1772,55 @@ gcm_prefs_remove_device (CcColorPanel *prefs, CdDevice *cd_device)
     } while (gtk_tree_model_iter_next (model, &iter));
 }
 
+static void
+gcm_prefs_update_device_list_extra_entry (CcColorPanel *prefs)
+{
+  CcColorPanelPrivate *priv = prefs->priv;
+  gboolean ret;
+  gchar *id_tmp;
+  gchar *title = NULL;
+  GtkTreeIter iter;
+
+  /* select the first device */
+  ret = gtk_tree_model_get_iter_first (GTK_TREE_MODEL (priv->list_store_devices), &iter);
+  if (!ret)
+    {
+      /* add the 'No devices detected' entry */
+      title = g_strdup_printf ("<i>%s</i>", _("No devices supporting color management detected"));
+      gtk_tree_store_append (priv->list_store_devices, &iter, NULL);
+      gtk_tree_store_set (priv->list_store_devices, &iter,
+                          GCM_PREFS_COLUMN_RADIO_VISIBLE, FALSE,
+                          GCM_PREFS_COLUMN_TITLE, title,
+                          -1);
+      g_free (title);
+      return;
+    }
+
+  /* remove the 'No devices detected' entry */
+  do
+    {
+      gtk_tree_model_get (GTK_TREE_MODEL (priv->list_store_devices), &iter,
+                          GCM_PREFS_COLUMN_DEVICE_PATH, &id_tmp,
+                          -1);
+      if (id_tmp == NULL)
+        {
+          gtk_tree_store_remove (priv->list_store_devices, &iter);
+          break;
+        }
+      g_free (id_tmp);
+    } while (gtk_tree_model_iter_next (GTK_TREE_MODEL (priv->list_store_devices), &iter));
+}
+
 static void
 gcm_prefs_device_added_cb (CdClient *client,
                            CdDevice *device,
                            CcColorPanel *prefs)
 {
-  /* remove the saved device if it's already there */
-  gcm_prefs_remove_device (prefs, device);
-
   /* add the device */
   gcm_prefs_add_device (prefs, device);
+
+  /* ensure we're not showing the 'No devices detected' entry */
+  gcm_prefs_update_device_list_extra_entry (prefs);
 }
 
 static void
@@ -1718,6 +1845,9 @@ gcm_prefs_device_removed_cb (CdClient *client,
   /* remove from the UI */
   gcm_prefs_remove_device (prefs, device);
 
+  /* ensure we showing the 'No devices detected' entry if required */
+  gcm_prefs_update_device_list_extra_entry (prefs);
+
   /* select the first device */
   ret = gtk_tree_model_get_iter_first (GTK_TREE_MODEL (priv->list_store_devices), &iter);
   if (!ret)
@@ -1732,6 +1862,17 @@ gcm_prefs_device_removed_cb (CdClient *client,
   gtk_tree_selection_select_iter (selection, &iter);
 }
 
+static gboolean
+gcm_prefs_tree_model_count_cb (GtkTreeModel *model,
+                               GtkTreePath *path,
+                               GtkTreeIter *iter,
+                               gpointer user_data)
+{
+  guint *i = (guint *) user_data;
+  (*i)++;
+  return FALSE;
+}
+
 static void
 gcm_prefs_get_devices_cb (GObject *object,
                           GAsyncResult *res,
@@ -1745,6 +1886,7 @@ gcm_prefs_get_devices_cb (GObject *object,
   GtkTreePath *path;
   GtkWidget *widget;
   guint i;
+  guint devices_and_profiles = 0;
   CcColorPanelPrivate *priv = prefs->priv;
 
   /* get devices and add them */
@@ -1762,12 +1904,24 @@ gcm_prefs_get_devices_cb (GObject *object,
       gcm_prefs_add_device (prefs, device);
     }
 
+  /* ensure we show the 'No devices detected' entry if empty */
+  gcm_prefs_update_device_list_extra_entry (prefs);
+
   /* set the cursor on the first device */
   widget = GTK_WIDGET (gtk_builder_get_object (priv->builder,
                                                "treeview_devices"));
   path = gtk_tree_path_new_from_string ("0");
   gtk_tree_view_set_cursor (GTK_TREE_VIEW (widget), path, NULL, FALSE);
   gtk_tree_path_free (path);
+
+  /* if we have only a few devices and profiles expand the treeview
+   * devices so they can all be seen */
+  gtk_tree_model_foreach (GTK_TREE_MODEL (priv->list_store_devices),
+                          gcm_prefs_tree_model_count_cb,
+                          &devices_and_profiles);
+  if (devices_and_profiles <= GCM_PREFS_MAX_DEVICES_PROFILES_EXPANDED)
+    gtk_tree_view_expand_all (GTK_TREE_VIEW (widget));
+
 out:
   if (devices != NULL)
     g_ptr_array_unref (devices);
@@ -1993,6 +2147,7 @@ gcm_prefs_connect_cb (GObject *object,
   /* set calibrate button sensitivity */
   gcm_prefs_sensor_coldplug (prefs);
 
+
   /* get devices */
   cd_client_get_devices (priv->client,
                          priv->cancellable,
