@@ -56,13 +56,20 @@ typedef struct {
   char *glx_renderer;
 } GraphicsData;
 
+typedef enum {
+	PK_NOT_AVAILABLE,
+	UPDATES_AVAILABLE,
+	UPDATES_NOT_AVAILABLE,
+	CHECKING_UPDATES
+} UpdatesState;
+
 struct _CcInfoPanelPrivate
 {
   GtkBuilder    *builder;
   char          *gnome_version;
   char          *gnome_distributor;
   char          *gnome_date;
-  gboolean       updates_available;
+  UpdatesState   updates_state;
   gboolean       is_fallback;
 
   /* Free space */
@@ -80,6 +87,7 @@ struct _CcInfoPanelPrivate
 };
 
 static void get_primary_disc_info_start (CcInfoPanel *self);
+static void refresh_update_button (CcInfoPanel *self);
 
 typedef struct
 {
@@ -560,19 +568,20 @@ query_done (GFile        *file,
 
   self->priv->cancellable = NULL;
   info = g_file_query_filesystem_info_finish (file, res, &error);
-  if (info == NULL)
+  if (info != NULL)
+    {
+      self->priv->total_bytes += g_file_info_get_attribute_uint64 (info, G_FILE_ATTRIBUTE_FILESYSTEM_SIZE);
+      g_object_unref (info);
+    }
+  else
     {
       char *path;
       path = g_file_get_path (file);
       g_warning ("Failed to get filesystem free space for '%s': %s", path, error->message);
       g_free (path);
       g_error_free (error);
-      return;
     }
 
-  self->priv->total_bytes += g_file_info_get_attribute_uint64 (info, G_FILE_ATTRIBUTE_FILESYSTEM_SIZE);
-  g_object_unref (info);
-
   /* And onto the next element */
   get_primary_disc_info_start (self);
 }
@@ -898,27 +907,12 @@ info_panel_setup_default_app (CcInfoPanel *self,
 {
   GtkWidget *button;
   GtkWidget *table;
-  GAppInfo  *info;
-  GError    *error = NULL;
   GtkWidget *label;
 
   table = WID ("default_apps_table");
 
-  /* FIXME: We need to do this because GtkAppChooser doesn't
-   * give us the opportunity to select what app should be selected
-   * by default.
-   * https://bugzilla.gnome.org/show_bug.cgi?id=642706 */
-  info = g_app_info_get_default_for_type (content_type, FALSE);
-  if (info != NULL && g_app_info_set_as_last_used_for_type (info, content_type, &error) == FALSE)
-    {
-      g_warning ("Failed to set '%s' as the default application for '%s': %s",
-                 g_app_info_get_name (info), content_type, error->message);
-      g_error_free (error);
-    }
-  if (info != NULL)
-    g_object_unref (info);
-
   button = gtk_app_chooser_button_new (content_type);
+  gtk_app_chooser_button_set_show_default_item (GTK_APP_CHOOSER_BUTTON (button), TRUE);
   gtk_table_attach (GTK_TABLE (table), button,
                     left_attach, right_attach,
                     top_attach, bottom_attach, GTK_FILL, 0, 0, 0);
@@ -1007,13 +1001,14 @@ info_panel_setup_selector (CcInfoPanel  *self)
 }
 
 static char *
-info_panel_get_hostname (CcInfoPanel  *self)
+get_hostname_property (CcInfoPanel *self,
+		       const char  *property)
 {
   GVariant *variant;
   char *str;
 
   variant = g_dbus_proxy_get_cached_property (self->priv->hostnamed_proxy,
-                                              "PrettyHostname");
+                                              property);
   if (!variant)
     {
       GError *error = NULL;
@@ -1023,14 +1018,14 @@ info_panel_get_hostname (CcInfoPanel  *self)
        * the property value when changing values */
       variant = g_dbus_proxy_call_sync (self->priv->hostnamed_proxy,
                                         "org.freedesktop.DBus.Properties.Get",
-                                        g_variant_new ("(ss)", "org.freedesktop.hostname1", "PrettyHostname"),
+                                        g_variant_new ("(ss)", "org.freedesktop.hostname1", property),
                                         G_DBUS_CALL_FLAGS_NONE,
                                         -1,
                                         NULL,
                                         &error);
       if (variant == NULL)
         {
-          g_warning ("Failed to get property 'PrettyHostname': %s", error->message);
+          g_warning ("Failed to get property '%s': %s", property, error->message);
           g_error_free (error);
           return NULL;
         }
@@ -1045,28 +1040,36 @@ info_panel_get_hostname (CcInfoPanel  *self)
       g_variant_unref (variant);
     }
 
+  return str;
+}
+
+static char *
+info_panel_get_hostname (CcInfoPanel  *self)
+{
+  char *str;
+
+  str = get_hostname_property (self, "PrettyHostname");
+
   /* Empty strings means that we need to fallback */
   if (str != NULL &&
       *str == '\0')
     {
       g_free (str);
-      str = NULL;
+      str = get_hostname_property (self, "Hostname");
     }
 
   return str;
 }
 
 static void
-text_changed_cb (GtkEntry        *entry,
-                 CcInfoPanel     *self)
+info_panel_set_hostname (CcInfoPanel *self,
+                         const char  *text)
 {
-  const char *text;
   char *hostname;
   GVariant *variant;
   GError *error = NULL;
 
-  text = gtk_entry_get_text (GTK_ENTRY (entry));
-
+  g_debug ("Setting PrettyHostname to '%s'", text);
   variant = g_dbus_proxy_call_sync (self->priv->hostnamed_proxy,
                                     "SetPrettyHostname",
                                     g_variant_new ("(sb)", text, FALSE),
@@ -1086,6 +1089,7 @@ text_changed_cb (GtkEntry        *entry,
   hostname = pretty_hostname_to_static (text, FALSE);
   g_assert (hostname);
 
+  g_debug ("Setting StaticHostname to '%s'", hostname);
   variant = g_dbus_proxy_call_sync (self->priv->hostnamed_proxy,
                                     "SetStaticHostname",
                                     g_variant_new ("(sb)", hostname, FALSE),
@@ -1103,20 +1107,37 @@ text_changed_cb (GtkEntry        *entry,
   g_free (hostname);
 }
 
+static void
+text_changed_cb (GtkEntry        *entry,
+                 CcInfoPanel     *self)
+{
+  const char *text;
+
+  text = gtk_entry_get_text (GTK_ENTRY (entry));
+  info_panel_set_hostname (self, text);
+}
+
 static void
 info_panel_setup_hostname (CcInfoPanel  *self,
                            GPermission  *permission)
 {
   char *str;
   GtkWidget *entry;
+  GError *error = NULL;
+
+  if (permission == NULL)
+    {
+      g_debug ("Will not show hostname, hostnamed not installed");
+      return;
+    }
 
   entry = WID ("name_entry");
 
-  gtk_widget_show (WID ("label4"));
-  gtk_widget_show (entry);
-
   if (g_permission_get_allowed (permission) != FALSE)
-    gtk_widget_set_sensitive (entry, TRUE);
+    {
+      g_debug ("Not allowed to change the hostname");
+      gtk_widget_set_sensitive (entry, TRUE);
+    }
 
   self->priv->hostnamed_proxy = g_dbus_proxy_new_for_bus_sync (G_BUS_TYPE_SYSTEM,
                                                                G_DBUS_PROXY_FLAGS_NONE,
@@ -1125,14 +1146,25 @@ info_panel_setup_hostname (CcInfoPanel  *self,
                                                                "/org/freedesktop/hostname1",
                                                                "org.freedesktop.hostname1",
                                                                NULL,
-                                                               NULL);
+                                                               &error);
 
   /* This could only happen if the policy file was installed
    * but not hostnamed, which points to a system bug */
-  g_assert (self->priv->hostnamed_proxy);
+  if (self->priv->hostnamed_proxy == NULL)
+    {
+      g_debug ("Couldn't get hostnamed to start, bailing: %s", error->message);
+      g_error_free (error);
+      return;
+    }
+
+  gtk_widget_show (WID ("label4"));
+  gtk_widget_show (entry);
 
   str = info_panel_get_hostname (self);
-  gtk_entry_set_text (GTK_ENTRY (entry), str);
+  if (str != NULL)
+    gtk_entry_set_text (GTK_ENTRY (entry), str);
+  else
+    gtk_entry_set_text (GTK_ENTRY (entry), "");
   g_free (str);
 
   g_signal_connect (G_OBJECT (entry), "changed",
@@ -1151,7 +1183,6 @@ info_panel_setup_overview (CcInfoPanel  *self)
 
   permission = polkit_permission_new_sync ("org.freedesktop.hostname1.set-static-hostname", NULL, NULL, NULL);
   /* Is hostnamed installed? */
-  if (permission != NULL)
     info_panel_setup_hostname (self, permission);
 
   res = load_gnome_version (&self->priv->gnome_version,
@@ -1190,6 +1221,8 @@ info_panel_setup_overview (CcInfoPanel  *self)
 
   widget = WID ("info_vbox");
   gtk_widget_reparent (widget, (GtkWidget *) self);
+
+  refresh_update_button (self);
 }
 
 static void
@@ -1198,10 +1231,27 @@ refresh_update_button (CcInfoPanel  *self)
   GtkWidget *widget;
 
   widget = WID ("updates_button");
-  if (self->priv->updates_available)
-    gtk_widget_show (widget);
-  else
-    gtk_widget_hide (widget);
+  if (widget == NULL)
+    return;
+
+  switch (self->priv->updates_state)
+    {
+      case PK_NOT_AVAILABLE:
+        gtk_widget_set_visible (widget, FALSE);
+        break;
+      case UPDATES_AVAILABLE:
+        gtk_widget_set_sensitive (widget, TRUE);
+        gtk_button_set_label (GTK_BUTTON (widget), _("Install Updates"));
+        break;
+      case UPDATES_NOT_AVAILABLE:
+        gtk_widget_set_sensitive (widget, FALSE);
+        gtk_button_set_label (GTK_BUTTON (widget), _("System Up-To-Date"));
+        break;
+      case CHECKING_UPDATES:
+        gtk_widget_set_sensitive (widget, FALSE);
+        gtk_button_set_label (GTK_BUTTON (widget), _("Checking for Updates"));
+        break;
+    }
 }
 
 static void
@@ -1213,7 +1263,7 @@ on_pk_transaction_signal (GDBusProxy *proxy,
 {
   if (g_strcmp0 (signal_name, "Package") == 0)
     {
-      self->priv->updates_available = TRUE;
+      self->priv->updates_state = UPDATES_AVAILABLE;
     }
   else if (g_strcmp0 (signal_name, "Finished") == 0)
     {
@@ -1257,7 +1307,8 @@ on_pk_get_tid_ready (GObject      *source,
   result = g_dbus_proxy_call_finish (G_DBUS_PROXY (source), res, &error);
   if (result == NULL)
     {
-      g_warning ("Error getting PackageKit transaction ID: %s", error->message);
+      if (g_error_matches (error, G_DBUS_ERROR, G_DBUS_ERROR_SERVICE_UNKNOWN) == FALSE)
+        g_warning ("Error getting PackageKit transaction ID: %s", error->message);
       g_error_free (error);
       return;
     }
@@ -1299,7 +1350,8 @@ on_pk_get_tid_ready (GObject      *source,
 static void
 refresh_updates (CcInfoPanel *self)
 {
-  self->priv->updates_available = FALSE;
+  self->priv->updates_state = CHECKING_UPDATES;
+  refresh_update_button (self);
 
   g_assert (self->priv->pk_proxy != NULL);
   g_dbus_proxy_call (self->priv->pk_proxy,
@@ -1364,7 +1416,10 @@ cc_info_panel_init (CcInfoPanel *self)
                                                         NULL,
                                                         NULL);
   if (self->priv->pk_proxy == NULL)
-    g_warning ("Unable to get PackageKit proxy object");
+    {
+      g_warning ("Unable to get PackageKit proxy object");
+      self->priv->updates_state = PK_NOT_AVAILABLE;
+    }
   else
     {
       g_signal_connect (self->priv->pk_proxy,
